/*
    kDriver - PC keyboard driver for the TI-89
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <alloc.h>
#include <dialogs.h>
#include <events.h>
#include <mem.h>
#include <statline.h>
#include <system.h>
#include <version.h>

#include "h220xtsr.h"
#include "array.h"

#if __TIGCC__==0&& __TIGCC_MINOR__< 94 && __TIGCC_BETA__ < 19
  #warning Old TIGCC Version -- Check tigcc.ticalc.org for updates
#endif

void _main(void)
{
  volatile HANDLE tsrHandle;
  char *pTsr;
  void *pHook = EV_hook - 16;

  if(EV_hook)
    while(pHook)
    {
      // 0x4576486B = EvHk
      if(0x4576486B != *(unsigned long*)pHook) break;

      if(!memcmp(pHook, tsr, 12)) {
        DlgMessage("kDriver", "kDriver is already installed", BT_OK, BT_NONE);
        return;
      }

      (unsigned long)pHook = *(unsigned long*)(pHook + 12) - 16;
    }

  if(!h220xTSR()) {
    DlgMessage("kDriver", "h220xTSR installation failed", BT_OK, BT_NONE);
    return;
  }

  if(!(tsrHandle = HeapAllocHigh(sizeof(tsr)))) {
    DlgMessage("kDriver", "Memory allocation failed", BT_OK, BT_NONE);
    return;
  }

  pTsr = HeapDeref(tsrHandle) + 0x40000;

  memcpy(pTsr, tsr, sizeof(tsr));

  *(unsigned long*)&pTsr[12] = (unsigned long)EV_hook;

  EX_patch(pTsr, pTsr + sizeof(tsr));
  ASM_fastcall(&pTsr[16]);
  EV_hook = (EVENT_HANDLER)&pTsr[16];

  ST_helpMsg("kDriver "VERSION" installed (C) SEB 2003");
}
